package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Eva.Assigns

Types and operations for the From plugin.

Datastructures and common operations for the results of the From plugin.

module DepsOrUnassigned : sig ... end
module Memory : sig ... end
type t = {
  1. return : Deps.t;
    (*

    Dependencies for the returned value

    *)
  2. memory : Memory.t;
    (*

    Dependencies on all the zones modified by the function

    *)
}
include Frama_c_kernel.Datatype.S with type t := t
include Frama_c_kernel.Datatype.S_no_copy with type t := t
val datatype_name : string

Unique name of the datatype.

val datatype_descr : t Frama_c_kernel.Descr.t

Datatype descriptor.

Packed version of the descriptor.

val reprs : t list

List of representants of the descriptor.

val equal : t -> t -> bool
val compare : t -> t -> int

Comparison: same spec than Stdlib.compare.

val hash : t -> int

Hash function: same spec than Hashtbl.hash.

val pretty : Format.formatter -> t -> unit

Pretty print each value in an user-friendly way.

val mem_project : (Frama_c_kernel.Project_skeleton.t -> bool) -> t -> bool

mem_project f x must return true iff there is a value p of type Project.t in x such that f p returns true.

val copy : t -> t

Deep copy: no possible sharing between x and copy x.

val top : t
val join : t -> t -> t

Extract the left part of a from result, ie. the zones that are written