package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Parameter Restrict.Context

type t
val top : t

The default context used in a top abstract state, or if no domain has been enabled — or no domain providing this context.

val narrow : t -> t -> t Eval.or_bottom

In a product of abstract domains, merges the context provided by the abstract state of each domain.