package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Eva.Unit_context

Empty context.

include Abstract_context.Leaf with type t = unit
include Abstract_context.S with type t = unit
type t = unit
val top : t

The default context used in a top abstract state, or if no domain has been enabled — or no domain providing this context.

val narrow : t -> t -> t Eval.or_bottom

In a product of abstract domains, merges the context provided by the abstract state of each domain.

The key identifies the module and the type t of context.