package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Db.PostdominatorsTypes

Declarations common to the various postdominators-computing modules

exception Top

Used for postdominators-related functions, when the postdominators of a statement cannot be computed. It means that there is no path from this statement to the function return.

module type Sig = sig ... end