package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module type Hptmap.Info

Required information for the correctness of the hptmaps.

type key
type v
val initial_values : (key * v) list list

List of the maps that must be shared between all instances of Frama-C (the maps being described by the list of their bindings). Must include all maps that are exported at Caml link-time when the functor is applied.

val dependencies : State.t list

Dependencies of the hash-consing table. The table will be cleared whenever one of those dependencies is cleared.