package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module type Hptset.Initial_values

type elt
val v : elt list list

List of the sets that must be shared between all instances of Frama-C (the sets being described by the list of their elements). Must include all sets that are exported at Caml link-time when the functor is applied. This usually includes at least the empty set, hence v nearly always contains [].