package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Mt_memory.TypesSource

Pointers are the adress of a variable, with a potential offset, and are used to refer in a simple way to an adress in memory

Sourcetype state_accesser =
  1. | Global
  2. | Local of functions_states
Sourceval map_functions_states_to_get_state : map_functions_states -> Frama_c_kernel.Cil_types.stmt -> state
Sourceval iter_requests : state_accesser -> Frama_c_kernel.Cil_types.stmt -> (Eva.Results.request -> unit) -> unit
Sourceval merge_map_non_map_functions_states : map_functions_states -> functions_states -> map_functions_states