package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Mt_types.RWSource

include Frama_c_kernel.Datatype.S with type t = rw
include Frama_c_kernel.Datatype.S_no_copy with type t = rw
val datatype_name : string

Unique name of the datatype.

val datatype_descr : t Frama_c_kernel.Descr.t

Datatype descriptor.

Packed version of the descriptor.

val reprs : t list

List of representants of the descriptor.

val equal : t -> t -> bool
val compare : t -> t -> int

Comparison: same spec than Stdlib.compare.

val hash : t -> int

Hash function: same spec than Hashtbl.hash.

val pretty : Format.formatter -> t -> unit

Pretty print each value in an user-friendly way.

val mem_project : (Frama_c_kernel.Project_skeleton.t -> bool) -> t -> bool

mem_project f x must return true iff there is a value p of type Project.t in x such that f p returns true.

val copy : t -> t

Deep copy: no possible sharing between x and copy x.

loc op returns the source location of the operation

Sourceval is_read : t -> bool

is_read op returns true if op is a read operation.

Sourceval pretty_op : Stdlib.Format.formatter -> t -> unit

Pretty-print the name of the operation (i.e. "read" or "write").

Sourceval pretty_loc : Stdlib.Format.formatter -> t -> unit

Pretty-print the source location of the operation and its callstack if applicable.