package frama-c

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file TacFilter.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
(**************************************************************************)
(*                                                                        *)
(*  This file is part of WP plug-in of Frama-C.                           *)
(*                                                                        *)
(*  Copyright (C) 2007-2023                                               *)
(*    CEA (Commissariat a l'energie atomique et aux energies              *)
(*         alternatives)                                                  *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

open Tactical

(* -------------------------------------------------------------------------- *)
(* --- Filter Tactical                                                    --- *)
(* -------------------------------------------------------------------------- *)

let vanti,panti =
  Tactical.checkbox ~id:"anti"
    ~title:"Absurd"
    ~descr:"Find Contradiction in Side Hypotheses"
    ~default:false ()

class filter =
  object(self)
    inherit Tactical.make ~id:"Wp.filter"
        ~title:"Filter"
        ~descr:"Dependent Erasure of Hypotheses"
        ~params:[panti]

    method select feedback _sel =
      let anti = self#get_field vanti in
      let process seq = ["Filter",Filtering.compute ~anti seq] in
      feedback#set_title (if anti then "Filter (absurd)" else "Filter") ;
      Applicable process

  end

let tactical = Tactical.export (new filter)

let strategy ?(priority=1.0) ?(anti=false) () =
  Strategy.{
    priority ; tactical ;
    selection = Empty ;
    arguments = [arg vanti anti] ;
  }